package dev.fyloz.colorrecipesexplorer.config

import dev.fyloz.colorrecipesexplorer.emergencyMode
import org.springframework.boot.context.properties.EnableConfigurationProperties
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.context.annotation.Profile
import org.springframework.core.env.Environment
import org.springframework.http.HttpMethod
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter
import org.springframework.security.config.http.SessionCreationPolicy
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.userdetails.UserDetails
import org.springframework.security.core.userdetails.UsernameNotFoundException
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.web.cors.CorsConfiguration
import org.springframework.web.cors.UrlBasedCorsConfigurationSource
import org.springframework.security.core.userdetails.User as SpringUser

@Configuration
@Profile("emergency")
@EnableConfigurationProperties(SecurityConfigurationProperties::class)
class EmergencySecurityConfig(
    val securityConfigurationProperties: SecurityConfigurationProperties,
    val environment: Environment
) : WebSecurityConfigurerAdapter() {
    init {
        emergencyMode = true
    }

    @Bean
    fun corsConfigurationSource() =
        UrlBasedCorsConfigurationSource().apply {
            registerCorsConfiguration("/**", CorsConfiguration().apply {
                allowedOrigins = listOf("http://localhost:4200")    // Angular development server
                allowedMethods = listOf(
                    HttpMethod.GET.name,
                    HttpMethod.POST.name,
                    HttpMethod.PUT.name,
                    HttpMethod.DELETE.name,
                    HttpMethod.OPTIONS.name,
                    HttpMethod.HEAD.name
                )
                allowCredentials = true
            }.applyPermitDefaultValues())
        }

    @Bean
    fun passwordEncoder() =
        BCryptPasswordEncoder()

    override fun configure(auth: AuthenticationManagerBuilder) {
        auth.inMemoryAuthentication()
            .withUser(securityConfigurationProperties.root!!.id.toString())
            .password(passwordEncoder().encode(securityConfigurationProperties.root!!.password))
            .authorities(SimpleGrantedAuthority("ADMIN"))
    }

    override fun configure(http: HttpSecurity) {
        val debugMode = "debug" in environment.activeProfiles

        http
            .headers().frameOptions().disable()
            .and()
            .csrf().disable()
            .addFilter(
                JwtAuthenticationFilter(
                    authenticationManager(),
                    securityConfigurationProperties
                ) { }
            )
            .addFilter(
                JwtAuthorizationFilter(
                    securityConfigurationProperties,
                    authenticationManager(),
                    this::loadUserById
                )
            )
            .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
            .and()
            .authorizeRequests()
            .antMatchers("**").permitAll()

        if (debugMode) {
            http
                .cors()
        }
    }

    private fun loadUserById(id: Long): UserDetails {
        if (id == securityConfigurationProperties.root!!.id) {
            return SpringUser(
                id.toString(),
                securityConfigurationProperties.root!!.password,
                listOf(SimpleGrantedAuthority("ADMIN"))
            )
        }
        throw UsernameNotFoundException(id.toString())
    }
}
